use std::sync::Arc;

use async_graphql::{
    async_stream::stream, futures_util::Stream, Context, Object, Subscription, ID,
};
use scel_core::App;

pub struct SubscriptionRoot;

struct DownloadChanged {
    id: ID,
}

#[Object]
impl DownloadChanged {
    async fn id(&self) -> &ID {
        &self.id
    }
}

#[Subscription]
impl SubscriptionRoot {
    async fn get_download(&self, ctx: &Context<'_>, id: ID) -> impl Stream<Item = DownloadChanged> {
        let app = ctx.data_unchecked::<Arc<App>>();

        let mut stream = app
            .download_service
            .subscribe_download(id.to_string())
            .await;

        stream! {
            while stream.changed().await.is_ok() {
                yield DownloadChanged {
                    id: id.clone()
                }
            }
        }
    }
}
